var defaultOpts = {
	//intervalles de dessin
	xMin: -10,
	xMax: 10,
	yMin: -10,
	yMax: 10,
	//Unités de graduation sur les axes
	gradX: 2,
	gradY: 1,
	longueurGrad: 5,
	//nombre de points à dessiner
	nbPoints: 100,
	epaisseur: 2,
	//marges à laisser à l'interieur du canvas.
	padHaut: 10,
	padBas: 10,
	padGauche: 10,
	padDroit: 10,
	//couleurs
	couleurAxes: 'black',
	couleurCourbe: function (i) {return '#' + (156<<(2*i)).toString(16);}
};

/**
 * Cré un object qui possède les propriétés et méthodes de o1 et o2. Celles de o2 écrasent les propriétés de même nom dans o1.
 * @param o1
 * @param o2
 */
function etend(o1, o2) {
	var o = {};
	for(var label in o1) {
		o[label] = o1[label];
	}
	for(label in o2) {
		o[label] = o2[label];
	}
	return o;
};

/**
 * Cré un nouveau repère à dessiner dans le canvas donné, avec les options données.
 */
var Repere = function(canvas, opts) {
	this.c = canvas;
	this.opts = etend(defaultOpts, opts);
	
	if (this.opts.xMin < this.opts.xMax && this.opts.yMin < this.opts.yMax) {
		this.fonctions = [];
	} else {
		console.log('erreur de dimension');
	}
	
};


function estEntre(val, min, max) {
	return (val >= min && val <= max);
}

var log = [], points = [];

Repere.prototype = {
	//Ajoute la fonction donnée pour dessin. Retourne l'indice. 
	ajouteFonction: function(f) {
		return this.fonctions.push(f) -1;
	},
	supprimeFonction: function(indice) {
		delete this.fonctions[indice];
	},
	assigneFonction: function(indice, f) {
		this.fonctions[indice] = f;
	},
	//converti les coordonnées math en coordonnées du canvas.
	canvasX: function(x) {
		//calcule la largeur effectives de dessin
		var w = this.c.width - this.opts.padGauche - this.opts.padDroit;
		return this.opts.padGauche + Math.floor((x - this.opts.xMin)/(this.opts.xMax - this.opts.xMin)*w);
	},
	canvasY: function(y){
		//hauteur effective
		var h = this.c.height - this.opts.padHaut - this.opts.padBas;
		return this.opts.padHaut + Math.floor(h + (this.opts.yMin-y)/(this.opts.yMax - this.opts.yMin)*h);
	},
	mathX : function(left) {
		var w = this.c.width - this.opts.padGauche - this.opts.padDroit;
		return this.opts.xMin + left/w*(this.opts.xMax - this.opts.xMin);
	},
	mathY: function(top) {
		var h = this.c.height - this.opts.padHaut - this.opts.padBas;
		return this.opts.yMin + (h - top + padHaut)/h*(this.opts.yMax - this.yMin);
	},
	_axes: function () {
		var i,x, y , ctx = this.c.getContext('2d');
		ctx.strokeStyle = this.opts.couleurAxes;
		ctx.beginPath();
		//dessine les axes sur le canvas.
		if (estEntre(0, this.opts.yMin, this.opts.yMax)) {
			//on ne dessine (Ox) que si y=0 es dans l'intervalle pour y.
			y = this.canvasY(0);
			ctx.moveTo(this.canvasX(this.opts.xMin), y);
			ctx.lineTo(this.canvasX(this.opts.xMax), y);
			for(i = Math.floor(this.opts.xMax); i >= this.opts.xMin; i -= this.opts.gradX) {
				x = this.canvasX(i);
				ctx.moveTo(x, y + this.opts.longueurGrad);
				ctx.lineTo(x, y - this.opts.longueurGrad);
			}
		}
		if (estEntre(0, this.opts.xMin, this.opts.xMax)) {
			//dessine (Oy)
			x = this.canvasX(0);
			ctx.moveTo(x, this.canvasY(this.opts.yMin));
			ctx.lineTo(x, this.canvasY(this.opts.yMax));
			for(i = Math.floor(this.opts.yMax); i >= this.opts.yMin; i-= this.opts.gradY) {
				y = this.canvasY(i);
				ctx.moveTo(x - this.opts.longueurGrad, y );
				ctx.lineTo(x + this.opts.longueurGrad, y );
			}
		}
		ctx.stroke();
	},
	_dessineFonction: function (indice, intervalle) {
		//dessine la i-ème fonction associée à ce repère. une fonction est un object possédant une méthode eval qui retrourne f(x).
		//intervalle est optionnel et doit être de la forme {debut: , fin}. il doit être compris dans {debut: xMin, fin: xMax}.
		var int = {debut: this.opts.xMin, fin: this.opts.xMax},
			ctx = this.c.getContext('2d'), x,y,
			dessin = false,
			f = this.fonctions[indice]; //a-t-on dessiné le dernier point calculé
		if (intervalle != undefined) {
			int = {debut: Math.max(intervalle.debut, this.opts.xMin), fin: Math.min(intervalle.fin, this.opts.xMax)};
		}
		ctx.strokeStyle = this.opts.couleurCourbe(indice);
		ctx.lineWidth = this.opts.epaisseur;
		ctx.beginPath();
		for (var i = 0; i < this.opts.nbPoints; i++) {
			x = int.debut + i*(int.fin - int.debut)/this.opts.nbPoints;
			y = f.eval(x);
			points.push([x,y]);
			if (estEntre(y, this.opts.yMin, this.opts.yMax)) {
				if (dessin) {
					ctx.lineTo(this.canvasX(x), this.canvasY(y));
					log.push([this.canvasX(x), this.canvasY(y)]);
				} else {
					dessin = true;
					ctx.moveTo(this.canvasX(x), this.canvasY(y));
				}
			} else {
				dessin = false;
			}
		}
		ctx.stroke();
		ctx.lineWidth = 1;
	},
	dessine: function (dessineAxe, callback, intervalle) {
		var ctx = this.c.getContext('2d');
		ctx.clearRect(0, 0, this.c.width, this.c.height);
		if (dessineAxe) {
			this._axes();
		}
		for(var indice in this.fonctions) {
			if (this.fonctions[indice] != undefined && this.fonctions[indice].eval != undefined) {
				this._dessineFonction(indice, intervalle);
			}
		}
		if (callback != undefined) {
			callback();
		}
	}
};